@@ -27,6 +27,7 @@ import apt_pkg
 import time
 import sys
 import rfc822
+from ConfigParser import ConfigParser
 from subprocess import Popen,PIPE
 
 class Dist(object):
@@ -40,24 +41,76 @@ class Dist(object):
         self.upgradeToolSig = None
 
 class MetaReleaseCore(object):
+    """
+    A MetaReleaseCore object astracts the list of released 
+    distributions. 
+    """
+
+    DEBUG = os.environ.has_key("DEBUG_UPDATE_MANAGER")
 
     # some constants
-    METARELEASE_URI = "http://changelogs.ubuntu.com/meta-release"
-    METARELEASE_URI_UNSTABLE = "http://changelogs.ubuntu.com/meta-release-development"
-    METARELEASE_URI_PROPOSED = "http://changelogs.ubuntu.com/meta-release-proposed"
-    def __init__(self, useDevelopmentRelease=False, useProposed=False):
-        # check what uri to use
-        if useDevelopmentRelease:
-            self.METARELEASE_URI = self.METARELEASE_URI_UNSTABLE
-        elif useProposed:
-            self.METARELEASE_URI = self.METARELEASE_URI_PROPOSED
+    CONF = "/etc/update-manager/release-upgrades"
+    CONF_METARELEASE = "/etc/update-manager/meta-release"
 
-        self._buildMetaReleaseFile()
-        self.metarelease_information = None
-        self.downloading = True
+    def __init__(self, 
+                 useDevelopmentRelease=False, 
+                 useProposed=False,
+                 forceLTS=False):
+        self._debug("MetaRelease.__init__() useDevel=%s useProposed=%s" % (useDevelopmentRelease, useProposed))
         # information about the available dists
+        self.downloading = True
         self.new_dist = None
         self.no_longer_supported = None
+
+        # default (if the conf file is missing)
+        self.METARELEASE_URI = "http://changelogs.ubuntu.com/meta-release"
+        self.METARELEASE_URI_LTS = "http://changelogs.ubuntu.com/meta-release-lts"
+        self.METARELEASE_URI_UNSTABLE_POSTFIX = "-development"
+        self.METARELEASE_URI_PROPOSED_POSTFIX = "-development"
+
+        # check the meta-release config first
+        parser = ConfigParser()
+        if os.path.exists(self.CONF_METARELEASE):
+            parser.read(self.CONF_METARELEASE)
+            # make changing the metarelease file and the location
+            # for the files easy
+            if parser.has_section("METARELEASE"):
+                sec = "METARELEASE"
+                for k in ["URI",
+                          "URI_LTS",
+                          "URI_UNSTABLE_POSTFIX",
+                          "URI_PROPOSED_POSTFIX"]:
+                    if parser.has_option(sec, k):
+                        self._debug("%s: %s " % (self.CONF_METARELEASE,
+                                                 parser.get(sec,k)))
+                        setattr(self, "%s_%s" % (sec, k), parser.get(sec, k))
+
+        # check the config file first to figure if we want lts upgrades only
+        parser = ConfigParser()
+        if os.path.exists(self.CONF):
+            parser.read(self.CONF)
+            # now check which specific url to use
+            if parser.has_option("DEFAULT","Prompt"):
+                type = parser.get("DEFAULT","Prompt").lower()
+                if (type == "never" or type == "no"):
+                    # nothing to do for this object
+                    # FIXME: what about no longer supported?
+                    self.downloading = False
+                    return
+                elif type == "lts":
+                    self.METARELEASE_URI = self.METARELEASE_URI_LTS
+        # needed for the _tryUpgradeSelf() code in DistUpgradeController
+        if forceLTS:
+            self.METARELEASE_URI = self.METARELEASE_URI_LTS
+        # devel and proposed "just" change the postfix
+        if useDevelopmentRelease:
+            self.METARELEASE_URI += self.METARELEASE_URI_UNSTABLE_POSTFIX
+        elif useProposed:
+            self.METARELEASE_URI += self.METARELEASE_URI_PROPOSED_POSTFIX
+
+        self._debug("metarelease-uri: %s" % self.METARELEASE_URI)
+        self._buildMetaReleaseFile()
+        self.metarelease_information = None
         # we start the download thread here and we have a timeout
         t=thread.start_new_thread(self.download, ())
         #t=thread.start_new_thread(self.check, ())
@@ -105,7 +158,7 @@ class MetaReleaseCore(object):
         return dist
     
     def parse(self):
-        #print "parse"
+        self._debug("MetaRelease.parse()")
         current_dist_name = self.get_dist()
         current_dist = None
         dists = []
@@ -146,13 +199,13 @@ class MetaReleaseCore(object):
         for dist in dists:
             if dist.date > current_dist.date and dist.supported == True: 
                 upgradable_to = dist
-                #print "new dist: %s" % upgradable_to
+                self._debug("new dist: %s" % upgradable_to)
                 break
 
         # only warn if unsupported and a new dist is available (because 
         # the development version is also unsupported)
         if upgradable_to != "" and not current_dist.supported:
-            self.dist_no_longer_supported(upgradabl_to)
+            self.dist_no_longer_supported(upgradable_to)
         elif upgradable_to != "":
             self.new_dist_available(upgradable_to)
 
@@ -162,7 +215,7 @@ class MetaReleaseCore(object):
     # the network thread that tries to fetch the meta-index file
     # can't touch the gui, runs as a thread
     def download(self):
-        #print "download"
+        self._debug("MetaRelease.download()")
         lastmodified = 0
         req = urllib2.Request(self.METARELEASE_URI)
         # make sure that we always get the latest file (#107716)
@@ -195,5 +248,17 @@ class MetaReleaseCore(object):
                 self.metarelease_information=open(self.METARELEASE_FILE,"r")
         # now check the information we have
         if self.metarelease_information != None:
+            self._debug("have self.metarelease_information")
             self.parse()
+        else:
+            self._debug("NO self.metarelease_information")
         self.downloading = False
+
+    def _debug(self, msg):
+        if self.DEBUG:
+            sys.stderr.write(msg+"\n")
+
+
+if __name__ == "__main__":
+    meta = MetaReleaseCore(False, False)
+    
