@@ -53,6 +53,7 @@ import subprocess
 import pwd
 import urllib2
 import httplib
+import socket
 import time
 import thread
 import xml.sax.saxutils
@@ -63,6 +64,7 @@ import dbus.service
 import dbus.glib
 
 from gettext import gettext as _
+from gettext import ngettext
 
 from Common.utils import *
 from Common.SimpleGladeApp import SimpleGladeApp
@@ -79,7 +81,7 @@ from MetaReleaseGObject import MetaRelease
 # - kill "all_changes" and move the changes into the "Update" class
 
 # list constants
-(LIST_CONTENTS, LIST_NAME, LIST_PKG) = range(3)
+(LIST_CONTENTS, LIST_NAME, LIST_PKG, LIST_ORIGIN) = range(4)
 
 # actions for "invoke_manager"
 (INSTALL, UPDATE) = range(2)
@@ -186,7 +188,9 @@ class MyCache(apt.Cache):
             srcrec = srcrecords.Lookup(srcpkg)
             if srcrec:
                 srcver = srcrecords.Version
-                if apt_pkg.VersionCompare(binver, srcver) > 0:
+                #if apt_pkg.VersionCompare(binver, srcver) > 0:
+                #    srcver = binver
+                if not srcver:
                     srcver = binver
                 #print "srcver: %s" % srcver
                 section = srcrecords.Section
@@ -206,7 +210,9 @@ class MyCache(apt.Cache):
         if srcpkg.startswith("lib"):
             prefix = "lib" + srcpkg[3]
 
-        # stip epoch
+        # stip epoch, but save epoch for later when displaying the
+        # launchpad changelog
+        srcver_epoch = srcver
         l = string.split(srcver,":")
         if len(l) > 1:
             srcver = "".join(l[1:])
@@ -244,15 +250,20 @@ class MyCache(apt.Cache):
             # Print an error if we failed to extract a changelog
             if len(alllines) == 0:
                 alllines = _("The list of changes is not available")
+
             # only write if we where not canceld
             if lock.locked():
                 self.all_changes[name] = [alllines, srcpkg]
-        except urllib2.HTTPError:
+        except urllib2.HTTPError, e:
             if lock.locked():
-                self.all_changes[name] = [_("The list of changes is not "
-                                            "available yet.\nPlease try again "
-                                            "later."), srcpkg]
-        except IOError, httplib.BadStatusLine:
+                self.all_changes[name] = [
+                    _("The list of changes is not available yet.\n\n"
+                      "Please use http://launchpad.net/ubuntu/+source/%s/%s/+changelog\n"
+                      "until the changes become available or try again "
+                      "later.") % (srcpkg, srcver_epoch),
+                    srcpkg]
+        except (IOError, httplib.BadStatusLine, socket.error), e:
+            print "caught exception: ", e
             if lock.locked():
                 self.all_changes[name] = [_("Failed to download the list "
                                             "of changes. \nPlease "
@@ -268,11 +279,18 @@ class UpdateList:
       self.importance = importance
       self.description = desc
 
-  def __init__(self):
+  def __init__(self, parent):
     # a map of packages under their origin
-    pipe = os.popen("lsb_release -c -s")
-    dist = pipe.read().strip()
-    del pipe
+    try:
+        pipe = os.popen("lsb_release -c -s")
+        dist = pipe.read().strip()
+        del pipe
+    except Exception, e:
+        print "Error in lsb_release: %s" % e
+        parent.error(_("Failed to detect distribution"),
+                     _("A error '%s' occurred while checking what system "
+                       "you are using.") % e)
+        sys.exit(1)
     self.distUpgradeWouldDelete = 0
     self.pkgs = {}
     self.num_updates = 0
@@ -299,11 +317,13 @@ class UpdateList:
     # do the upgrade
     self.distUpgradeWouldDelete = cache.saveDistUpgrade()
 
+    dselect_upgrade_origin = self.UpdateOrigin(_("Previous selected"), 1)
+
     # sort by origin
     for pkg in cache:
       if pkg.isUpgradable or pkg.markedInstall:
         if pkg.candidateOrigin == None:
-            # can happen for e.g. loged packages
+            # can happen for e.g. locked packages
             # FIXME: do something more sensible here (but what?)
             print "WARNING: upgradable but no canidateOrigin?!?: ", pkg.name
             continue
@@ -364,7 +384,8 @@ class UpdateManager(SimpleGladeApp):
     self.button_close.connect("clicked", lambda w: self.exit())
 
     # the treeview (move into it's own code!)
-    self.store = gtk.ListStore(str, str, gobject.TYPE_PYOBJECT)
+    self.store = gtk.ListStore(str, str, gobject.TYPE_PYOBJECT, 
+                               gobject.TYPE_PYOBJECT)
     self.treeview_update.set_model(self.store)
     self.treeview_update.set_headers_clickable(True);
 
@@ -379,7 +400,6 @@ class UpdateManager(SimpleGladeApp):
     column_install = gtk.TreeViewColumn("Install", cr)
     column_install.set_cell_data_func (cr, self.install_column_view_func)
     column = gtk.TreeViewColumn("Name", tr, markup=LIST_CONTENTS)
-    column.set_cell_data_func (tr, self.package_column_view_func)
     column.set_resizable(True)
     major,minor,patch = gtk.pygtk_version
     if (major >= 2) and (minor >= 5):
@@ -387,13 +407,14 @@ class UpdateManager(SimpleGladeApp):
       column_install.set_fixed_width(30)
       column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
       column.set_fixed_width(100)
-      #self.treeview_update.set_fixed_height_mode(True)
+      self.treeview_update.set_fixed_height_mode(True)
 
     self.treeview_update.append_column(column_install)
     column_install.set_visible(True)
     self.treeview_update.append_column(column)
     self.treeview_update.set_search_column(LIST_NAME)
     self.treeview_update.connect("button-press-event", self.show_context_menu)
+    self.treeview_update.connect("row-activated", self.row_activated)
 
 
 
@@ -404,56 +425,13 @@ class UpdateManager(SimpleGladeApp):
         self.button_help.set_sensitive(False)
 
     self.gconfclient = gconf.client_get_default()
-    self.init_proxy()
+    init_proxy(self.gconfclient)
 
     # restore state
     self.restore_state()
     self.window_main.show()
 
-  def init_proxy(self):
-      # proxy settings, first check for http_proxy environment (always wins),
-      # then look into synaptics conffile, then into gconf 
-      if os.getenv("http_proxy"):
-          return
-      SYNAPTIC_CONF_FILE = "%s/.synaptic/synaptic.conf" % pwd.getpwuid(0)[5]
-      proxy = None
-      if os.path.exists(SYNAPTIC_CONF_FILE):
-          cnf = apt_pkg.newConfiguration()
-          apt_pkg.ReadConfigFile(cnf, SYNAPTIC_CONF_FILE)
-          use_proxy = cnf.FindB("Synaptic::useProxy", False)
-          if use_proxy:
-              proxy_host = cnf.Find("Synaptic::httpProxy")
-              proxy_port = str(cnf.FindI("Synaptic::httpProxyPort"))
-              if proxy_host and proxy_port:
-                  # FIXME: set the proxy for libapt here as well (e.g. for the
-                  #        DistUpgradeFetcher
-                  proxy = "http://%s:%s/" % (proxy_host, proxy_port)
-      elif self.gconfclient.get_bool("/system/http_proxy/use_http_proxy"):
-          host = self.gconfclient.get_string("/system/http_proxy/host")
-          port = self.gconfclient.get_int("/system/http_proxy/port")
-          use_auth = self.gconfclient.get_bool("/system/http_proxy/use_authentication")
-          if host and port:
-              if use_auth:
-                  auth_user = self.gconfclient.get_string("/system/http_proxy/authentication_user")
-                  auth_pw = self.gconfclient.get_string("/system/http_proxy/authentication_password")
-                  proxy = "http://%s:%s@%s:%s/" % (auth_user,auth_pw,host, port)
-              else:
-                  proxy = "http://%s:%s/" % (host, port)
-      if proxy:
-          proxy_support = urllib2.ProxyHandler({"http":proxy})
-          opener = urllib2.build_opener(proxy_support)
-          urllib2.install_opener(opener)
-          os.putenv("http_proxy",proxy)
-
-  def header_column_func(self, cell_layout, renderer, model, iter):
-    pkg = model.get_value(iter, LIST_PKG)
-    if pkg == None:
-      renderer.set_property("sensitive", False)
-    else:
-      renderer.set_property("sensitive", True)
-
   def install_column_view_func(self, cell_layout, renderer, model, iter):
-    self.header_column_func(cell_layout, renderer, model, iter)
     pkg = model.get_value(iter, LIST_PKG)
     # hide it if we are only a header line
     renderer.set_property("visible", pkg != None)
@@ -466,9 +444,6 @@ class UpdateManager(SimpleGladeApp):
     else: 
         renderer.set_property("activatable", True)
 
-  def package_column_view_func(self, cell_layout, renderer, model, iter):
-    self.header_column_func(cell_layout, renderer, model, iter)
-      
   def setupDbus(self):
     """ this sets up a dbus listener if none is installed alread """
     # check if there is another g-a-i already and if not setup one
@@ -676,9 +651,13 @@ class UpdateManager(SimpleGladeApp):
       ago_days = int( (time.time() - mtime) / (24*60*60))
       ago_hours = int((time.time() - mtime) / (60*60) )
       if ago_days > 0:
-          return _("The package information was last updated %s days ago.") % ago_days
+          return ngettext("The package information was last updated %s day ago.",
+                          "The package information was last updated %s days ago.",
+                          ago_days) % ago_days
       elif ago_hours > 0:
-          return _("The package information was last updated %s hours ago.") % ago_hours
+          return ngettext("The package information was last updated %s hour ago.",
+                          "The package information was last updated %s hours ago.",
+                          ago_hours) % ago_hours
       else:
           return _("The package information was last updated less than one hour ago.")
       return None
@@ -703,10 +682,10 @@ class UpdateManager(SimpleGladeApp):
               text_label_main = self._get_last_apt_get_update_text()
       else:
           text_header = "<big><b>%s</b></big>" % \
-                        (gettext.ngettext("You can install %s update.",
-                                          "You can install %s updates.", 
-                                          num_updates) % \
-                                          num_updates)
+                        (ngettext("You can install %s update.",
+                                  "You can install %s updates.", 
+                                  num_updates) % \
+                                  num_updates)
           text_download = _("Download size: %s") % humanize_size(self.dl_size)
           self.notebook_details.set_sensitive(True)
           self.treeview_update.set_sensitive(True)
@@ -740,8 +719,9 @@ class UpdateManager(SimpleGladeApp):
       # close when update was successful (its ok to use a Synaptic::
       # option here, it will not get auto-saved, because synaptic does
       # not save options in non-interactive mode)
-      cmd.append("-o")
-      cmd.append("Synaptic::closeZvt=true")
+      if self.gconfclient.get_bool("/apps/update-manager/autoclose_install_window"):
+          cmd.append("-o")
+          cmd.append("Synaptic::closeZvt=true")
       # custom progress strings
       cmd.append("--progress-str")
       cmd.append("%s" % _("Please wait, this can take some time."))
@@ -808,6 +788,32 @@ class UpdateManager(SimpleGladeApp):
     self.window_main.set_sensitive(True)
     self.window_main.window.set_cursor(None)
 
+  def row_activated(self, treeview, path, column):
+      print self.list.held_back
+      iter = self.store.get_iter(path)
+      pkg = self.store.get_value(iter, LIST_PKG)
+      origin = self.store.get_value(iter, LIST_ORIGIN)
+      if pkg is not None:
+          return
+      self.setBusy(True)
+      actiongroup = apt_pkg.GetPkgActionGroup(self.cache._depcache)
+      for pkg in self.list.pkgs[origin]:
+          if pkg.markedInstall or pkg.markedUpgrade:
+              #print "marking keep: ", pkg.name
+              pkg.markKeep()
+          elif not (pkg.name in self.list.held_back):
+              #print "marking install: ", pkg.name
+              pkg.markInstall(autoFix=False,autoInst=False)
+      # check if we left breakage
+      if self.cache._depcache.BrokenCount:
+          Fix = apt_pkg.GetPkgProblemResolver(self.cache._depcache)
+          Fix.ResolveByKeep()
+      self.refresh_updates_count()
+      self.treeview_update.queue_draw()
+      del actiongroup
+      self.setBusy(False)
+
+
   def toggled(self, renderer, path):
     """ a toggle button in the listview was toggled """
     iter = self.store.get_iter(path)
@@ -876,7 +882,7 @@ class UpdateManager(SimpleGladeApp):
     except SystemError, e:
         msg = ("<big><b>%s</b></big>\n\n%s\n'%s'" %
                (_("Could not initialize the package information"),
-                _("A unresolvable problem occurred while "
+                _("An unresolvable problem occurred while "
                   "initializing the package information.\n\n"
                   "Please report this bug against the 'update-manager' "
                   "package and include the following error message:\n"),
@@ -891,14 +897,14 @@ class UpdateManager(SimpleGladeApp):
         dialog.destroy()
         sys.exit(1)
     self.store.clear()
-    self.list = UpdateList()
+    self.list = UpdateList(self)
     # fill them again
     try:
         self.list.update(self.cache)
     except SystemError, e:
         msg = ("<big><b>%s</b></big>\n\n%s\n'%s'" %
                (_("Could not calculate the upgrade"),
-                _("A unresolvable problem occurred while "
+                _("An unresolvable problem occurred while "
                   "calculating the upgrade.\n\n"
                   "Please report this bug against the 'update-manager' "
                   "package and include the following error message:"),
@@ -917,15 +923,24 @@ class UpdateManager(SimpleGladeApp):
       origin_list.reverse()
       for origin in origin_list:
         self.store.append(['<b><big>%s</big></b>' % origin.description,
-                           origin.description, None])
+                           origin.description, None, origin])
         for pkg in self.list.pkgs[origin]:
           name = xml.sax.saxutils.escape(pkg.name)
           summary = xml.sax.saxutils.escape(pkg.summary)
           contents = "<b>%s</b>\n<small>%s</small>" % (name, summary)
           #TRANSLATORS: the b stands for Bytes
           size = _("(Size: %s)") % humanize_size(pkg.packageSize)
-          contents = "%s <small>%s</small>" % (contents, size)
-          self.store.append([contents, pkg.name, pkg])
+          if pkg.installedVersion != None:
+              version = _("From version %(old_version)s to %(new_version)s") %\
+                  {"old_version" : pkg.installedVersion,
+                   "new_version" : pkg.candidateVersion}
+          else:
+              version = _("Version %s") % pkg.candidateVersion
+          if self.gconfclient.get_bool("/apps/update-manager/show_versions"):
+              contents = "%s\n<small>%s %s</small>" % (contents, version, size)
+          else:
+              contents = "%s <small>%s</small>" % (contents, size)
+          self.store.append([contents, pkg.name, pkg, None])
     self.update_count()
     self.setBusy(False)
     self.check_all_updates_installable()
@@ -946,6 +961,17 @@ class UpdateManager(SimpleGladeApp):
     dialog.run()
     dialog.destroy()
 
+  def error(self, summary, details):
+      " helper function to display a error message "
+      msg = ("<big><b>%s</b></big>\n\n%s\n" % (summary, details) )
+      dialog = gtk.MessageDialog(self.window_main,
+                                 0, gtk.MESSAGE_ERROR,
+                                 gtk.BUTTONS_CLOSE,"")
+      dialog.set_markup(msg)
+      dialog.vbox.set_spacing(6)
+      dialog.run()
+      dialog.destroy()
+
   def on_button_dist_upgrade_clicked(self, button):
       #print "on_button_dist_upgrade_clicked"
       progress = GtkProgress.GtkFetchProgress(self,
